"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var path_to_regexp_1 = require("path-to-regexp");
var paths_1 = __importDefault(require("../paths"));
var utils_1 = __importDefault(require("../utils"));
function readMockFile(req, res, next, mockConfig) {
    if (!mockConfig) {
        return new Error('没有找到mock配置');
    }
    try {
        var joinPath = paths_1.default.getJoinPath(mockConfig.rootDir, mockConfig.api, req.path);
        var reqUrl_1 = path_1.default.join(mockConfig.api, req.url);
        if (mockConfig.whiteList
            && Array.isArray(mockConfig.whiteList)
            && mockConfig.whiteList.indexOf(reqUrl_1) > -1) {
            var isWhite = mockConfig.whiteList.some(function (white) {
                var matchExp = path_to_regexp_1.match(white);
                return matchExp(reqUrl_1);
            });
            if (isWhite) {
                return next();
            }
        }
        var mock = getMockResponse(joinPath);
        if (utils_1.default.isFunction(mock)) {
            var ret = mock(req, res);
            // 如果返回的是object那么自动处理这个函数 如果函数内处理了就不要返回值
            if (utils_1.default.isObject(ret)) {
                res.send(ret);
            }
        }
        else if (mock !== undefined && mock !== false) {
            res.send(mock);
        }
        else {
            next();
        }
    }
    catch (e) {
        utils_1.default.error(e);
    }
}
function readJsFile(filePath) {
    try {
        var parentDir_1 = path_1.default.resolve(filePath, '../');
        var files = fs_1.default.readdirSync(parentDir_1);
        return files
            .filter(function (filename) { return /\.js$/.test(filename); })
            .map(function (filename) { return path_1.default.resolve(parentDir_1, filename); });
    }
    catch (e) {
        return [];
    }
}
function getMockResponse(joinPath) {
    var mock;
    try {
        clearRequireCache(joinPath + ".js");
        mock = utils_1.default.requireModule(joinPath + ".js");
    }
    catch (e) {
        // 动态路由 例如 /test/1 => /test/:id
        var jsFiles = readJsFile(joinPath);
        var i = 0;
        var fileIndex = -1;
        var len = jsFiles.length;
        while (i < len) {
            var jsFile = jsFiles[i];
            var matchExp = path_to_regexp_1.match(jsFile.replace(/\.js$/, ''), { sensitive: true });
            var realFileInfo = matchExp(joinPath);
            if (realFileInfo) {
                fileIndex = i;
                break;
            }
            i++;
        }
        if (fileIndex > -1) {
            clearRequireCache(jsFiles[fileIndex]);
            mock = require(jsFiles[fileIndex]); // 异步过程
        }
    }
    return mock;
}
function clearRequireCache(filePath) {
    var cacheMod = require.cache[filePath];
    // 防止内存泄漏
    if (cacheMod && cacheMod.parent) {
        var index = cacheMod.parent.children.indexOf(cacheMod);
        if (index > -1) {
            cacheMod.parent.children.splice(index, 1);
        }
    }
    delete require.cache[filePath];
}
exports.default = {
    middleware: function (options) {
        return function (req, res, next) {
            readMockFile(req, res, next, options);
        };
    }
};
