"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("./babelRegister");
var webpack_1 = __importDefault(require("webpack"));
var webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
var utils_1 = __importDefault(require("./utils"));
var clearConsole_1 = __importDefault(require("./clearConsole"));
var getConfig_1 = __importDefault(require("./getConfig"));
var compilerErrorHandle_1 = __importDefault(require("./compilerErrorHandle"));
function runServer(options) {
    var webpackConfig = getConfig_1.default(options).webpack;
    var devServerConfig = webpackConfig.devServer || {};
    var URL = "http://" + devServerConfig.host + ":" + devServerConfig.port;
    var compiler = webpack_1.default(webpackConfig);
    var server = new webpack_dev_server_1.default(compiler, devServerConfig);
    var isFirstCompiler = true;
    compiler.hooks.beforeCompile.tap('beforeCompile', function () {
        if (isFirstCompiler) {
            return;
        }
        clearConsole_1.default();
        utils_1.default.success(utils_1.default.chalk.green('Rebuild start...'));
    });
    compiler.hooks.done.tap('dev server', function (stats) {
        if (isFirstCompiler) {
            var proxyPaths = utils_1.default.getWebpackProxyPath(webpackConfig);
            utils_1.default.success(utils_1.default.chalk.green('Local:') + "  " + URL);
            if (proxyPaths.length > 0) {
                utils_1.default.success(utils_1.default.chalk.green('Proxy:') + "  [" + proxyPaths.join(', ') + "]");
            }
            utils_1.default.success(utils_1.default.chalk.green('Compiled successfully!'));
            isFirstCompiler = false;
        }
        else {
            utils_1.default.success(utils_1.default.chalk.green('Compiled successfully!'));
        }
        compilerErrorHandle_1.default(stats, null, isFirstCompiler);
    });
    if (!devServerConfig.port || !devServerConfig.host) {
        utils_1.default.error("port: " + devServerConfig.port + ", host: " + devServerConfig.host);
        process.exit(0);
    }
    server.listen(devServerConfig.port, devServerConfig.host, function (e) {
        if (isFirstCompiler) {
            clearConsole_1.default();
        }
        utils_1.default.success(utils_1.default.chalk.green('Server starting...'));
    }).on('error', function (e) {
        utils_1.default.error(e.message);
        process.exit(0);
    });
}
exports.default = runServer;
;
