import './babelRegister';
import fs from 'fs';
import path from 'path';
import bytes from 'bytes';
import rimraf from 'rimraf';
import webpack from 'webpack';
import gzipSize from 'gzip-size';

import getConfig from './getConfig';
import compilerErrorHandle from './compilerErrorHandle';

import utils from './utils';

export default function runBuild() {
    const webpackConfig: webpack.Configuration = getConfig().webpack;

    utils.success('Creating an optimized production build...');
    // 删除build路径
    rimraf.sync(webpackConfig?.output?.path || '');

    webpack(webpackConfig, function (err: Error, stats: webpack.Stats) {
        compilerErrorHandle(stats, err);
        stats?.toJson()?.assets?.filter(asset => {
            if (!/(.css|.js|.html)/.test(asset.name || '')) {
                return;
            }
            const filePath = path.resolve(webpackConfig.output?.path || '', asset.name);
            if (fs.existsSync(filePath)) {
                const fileContents = fs.readFileSync(filePath);
                const size = gzipSize.sync(fileContents);
                utils.loggerPrefix(
                    utils.chalk.green(`${asset.name} ${size > 0
                        ? `${utils.chalk.cyan('===>')} ${utils.chalk.blue(bytes(size))}`
                        : ''
                        }`)
                );
            }
        });

        utils.success('Compiled successfully!');
    });
};
