import {Request, Response, NextFunction} from 'express';
import utils from '../utils';

export default function loggerMiddleware() {
    return function (req: Request, res: Response, next: NextFunction) {
        if (
            /\.(css|css\?\d+|js|png|jpg|svg|gif|html|text|doc|ico|hot-update.json)$/.test(req.url)
            || req.url === '/'
        ) {
            return next();
        }
        const startDate = Date.now();
        next();

        const msg = `[${req.method}] ${req.url}  ${res.statusCode} ${Date.now() - startDate}ms`;
        if (res.statusCode >= 400) {
            utils.warning(msg);
        }
        else {
            utils.logger(msg);
        }
    };
};
