import './babelRegister';

import webpack from 'webpack';
import {Options} from './start';
import WebpackDevServer from 'webpack-dev-server';

import utils from './utils';
import clearConsole from './clearConsole';
import getConfig from './getConfig';
import compilerErrorHandle from './compilerErrorHandle';

export default function runServer(options: Options) {
    const webpackConfig: webpack.Configuration = getConfig(options).webpack;
    const devServerConfig = webpackConfig.devServer || {};

    const URL = `http://${devServerConfig.host}:${devServerConfig.port}`;
    const compiler = webpack(webpackConfig);
    const server = new WebpackDevServer(compiler, devServerConfig);
    let isFirstCompiler = true;

    compiler.hooks.beforeCompile.tap('beforeCompile', function () {
        if (isFirstCompiler) {
            return;
        }

        clearConsole();
        utils.success(utils.chalk.green('Rebuild start...'));
    });

    compiler.hooks.done.tap('dev server', function (stats) {
        if (isFirstCompiler) {
            const proxyPaths = utils.getWebpackProxyPath(webpackConfig);

            utils.success(`${utils.chalk.green('Local:')}  ${URL}`);
            if (proxyPaths.length > 0) {
                utils.success(`${utils.chalk.green('Proxy:')}  [${proxyPaths.join(', ')}]`);
            }
            utils.success(utils.chalk.green('Compiled successfully!'));

            isFirstCompiler = false;
        }
        else {
            utils.success(utils.chalk.green('Compiled successfully!'));
        }

        compilerErrorHandle(stats, null, isFirstCompiler);
    });

    if (!devServerConfig.port || !devServerConfig.host) {
        utils.error(`port: ${devServerConfig.port}, host: ${devServerConfig.host}`);
        process.exit(0);
    }

    server.listen(devServerConfig.port, devServerConfig.host, function (e) {
        if (isFirstCompiler) {
            clearConsole();
        }
        utils.success(utils.chalk.green('Server starting...'));
    }).on('error', function (e: Error) {
        utils.error(e.message);
        process.exit(0);
    });
};
